#ifndef DEBUGSTREAM_H
#define DEBUGSTREAM_H

// Tracestream implementation
//
//	Tested on: 
//            CodeGear (former Borland): 
//              - Turbo C++     (bcc32 v. 5.82 )
//              - Free compiler (bcc32 v. 5.5.1)
//            mingw 3.4.5 - (Unicode version of stream crashes)
//            Visual C++ 2003
//            Visual C++ 2005 SP1
//            Visual C++ 2008
//
// Version:	1.2
//
// Created:  01/12/2006
// Modified: 07/01/2007 - optimized buffer usage
//           19/10/2008 - added singleton implementation, minor fixes
//
//	Author:		Darko Miletic	kiklop@fibertel.com.ar
//
//	Disclaimer
//	----------
//	THIS SOFTWARE AND THE ACCOMPANYING FILES ARE DISTRIBUTED "AS IS" AND WITHOUT
//	ANY WARRANTIES WHETHER EXPRESSED OR IMPLIED. NO REPONSIBILITIES FOR POSSIBLE
//	DAMAGES OR EVEN FUNCTIONALITY CAN BE TAKEN. THE USER MUST ASSUME THE ENTIRE
//	RISK OF USING THIS SOFTWARE.
//
//	Terms of use
//	------------
//	THIS SOFTWARE IS FREE FOR BOTH PERSONAL AND COMMERCIAL USE

#include <cstddef  >
#include <windows.h>
#include <tchar.h  >
#include <streambuf>
#include <ostream  >
#include <string   >

#ifdef __BORLANDC__
#pragma warn -inl
#endif

template < typename charT >
inline void DumpTraceText_(const charT *ptr) {
    ::OutputDebugString(ptr);
}

template < >
inline void DumpTraceText_(const char *ptr) {
    ::OutputDebugStringA(ptr);
}

template < >
inline void DumpTraceText_(const wchar_t *ptr) {
    ::OutputDebugStringW(ptr);
}

template < typename charT, class traits = std::char_traits<charT> >
class debuglog_streambuf : public std::basic_streambuf<charT,traits> {
private:
    typedef std::basic_string<charT,traits> buffer_type;
    buffer_type buffer_;
    typedef typename std::basic_streambuf<charT, traits>::int_type int_type;
protected:
    virtual int_type overflow(int_type c__chv_ = traits::eof()) {
        if (!traits::eq_int_type(c__chv_, traits::eof())) {
            buffer_.push_back(traits::to_char_type(c__chv_));
        }

        return traits::not_eof(c__chv_);
    }
    virtual std::streamsize xsputn(const charT *s__buffc_,
                                   std::streamsize i__numv_) {
        buffer_.append(s__buffc_,static_cast<typename buffer_type::size_type>(i__numv_));
        return i__numv_;
    }
    virtual int sync() {
        DumpTraceText_(buffer_.c_str());
        buffer_.clear();
        return 0;
    }
public:
    debuglog_streambuf() : buffer_() {
        buffer_.reserve(250);
    }
    ~debuglog_streambuf() {
        sync();
    }
private:
    debuglog_streambuf(const debuglog_streambuf&);
    debuglog_streambuf& operator=(const debuglog_streambuf&);
};

template< typename charT, class traits = std::char_traits<charT> >
struct debuglog_init {
private:
    debuglog_streambuf<charT, traits> buf_;
    debuglog_init(const debuglog_init&);
    debuglog_init& operator=(const debuglog_init&);

public:
    debuglog_init() {}
    debuglog_streambuf<charT, traits>* buf()
    {
        return &buf_;
    }
};

template < typename charT, class traits = std::char_traits<charT> >
class debuglog_stream : private debuglog_init<charT,traits>,
            public std::basic_ostream<charT,traits>
{
private:
    debuglog_streambuf<charT,traits> buff_;
    typedef debuglog_init<charT, traits> dbgbuf_init;
public:
    debuglog_stream() : dbgbuf_init(), std::basic_ostream<charT,traits>(dbgbuf_init::buf()) {}
private:
    debuglog_stream(const debuglog_stream&);
    debuglog_stream& operator=(const debuglog_stream&);
};

typedef debuglog_stream<TCHAR  > _ttracestream;
typedef debuglog_stream<char   > tracestream  ;
typedef debuglog_stream<wchar_t> wtracestream ;

template < typename charT, class traits = std::char_traits<charT> >
class dstream {
    typedef debuglog_stream<typename charT, typename traits> streamType;
public:
    static streamType& dout(void) {
        static dstream streamInst;
        return streamInst.stream_;
    }
#ifndef __BORLANDC__
private:
#endif /* __BORLANDC__ */
    ~dstream(void) {}
private:
    dstream(void) {}
    dstream(dstream const&);
    dstream& operator=(dstream const&);
    streamType stream_;
};

typedef dstream<char   > cdstream;
typedef dstream<wchar_t> wdstream;
typedef dstream<TCHAR  > tdstream;

#endif /* DEBUGSTREAM_H */
